@@ -65,7 +65,8 @@ typedef struct {
 static struct option    options[] = {
         /* name, has-arg, string-return-val, char-return-val */
         {"help",        0, NULL, 'h'},
-        {"interactive", 0, NULL, 'i'},
+        {"list",        0, NULL, 'l'},
+        {"machine",     0, NULL, 'm'},
         {"script",      0, NULL, 's'},
         {"version",     0, NULL, 'v'},
         {NULL,          0, NULL, 0}
@@ -74,13 +75,15 @@ static struct option    options[] = {
 
 static char*    options_help [][2] = {
         {"help",        N_("displays this help message")},
-        {"interactive", N_("where necessary, prompts for user intervention")},
+        {"list",        N_("lists partition tables of all detected devices")},
+        {"machine",     N_("displays machine parseable output")},
         {"script",      N_("never prompts for user intervention")},
         {"version",     N_("displays the version")},
         {NULL,          NULL}
 };
 
-int     opt_script_mode;
+int     opt_script_mode = 0;
+int     opt_machine_mode = 0;
 int     is_toggle_mode = 0;
 
 static char* number_msg = N_(
@@ -107,9 +110,9 @@ static char* copyright_msg = N_(
 "Copyright (C) 1998 - 2006 Free Software Foundation, Inc.\n"
 "This program is free software, covered by the GNU General Public License.\n"
 "\n"
-"This program is distributed in the hope that it will be useful, "
-"but WITHOUT ANY WARRANTY; without even the implied warranty of "
-"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the "
+"This program is distributed in the hope that it will be useful,\n"
+"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
 "GNU General Public License for more details.\n\n");
 
 static char* label_type_msg;
@@ -124,6 +127,7 @@ static Command* commands [256] = {NULL};
 static PedTimer* timer;
 static TimerContext timer_context;
 
+static int _print_list (int cli);
 static void _done (PedDevice* dev);
 
 static void
@@ -161,37 +165,59 @@ _timer_handler (PedTimer* timer, void* context)
 static int
 _partition_warn_busy (PedPartition* part)
 {
-        char* path = ped_partition_get_path (part);
+        char* path;
 
         if (ped_partition_is_busy (part)) {
+                path = ped_partition_get_path (part);
                 ped_exception_throw (
                         PED_EXCEPTION_ERROR,
                         PED_EXCEPTION_CANCEL,
-                        _("Partition %s is being used.  You must unmount it "
+                        _("Partition %s is being used. You must unmount it "
                           "before you modify it with Parted."),
                         path);
                 ped_free (path);
                 return 0;
         }
-        ped_free (path);
         return 1;
 }
 
 static int
 _disk_warn_busy (PedDisk* disk)
 {
-        if (ped_device_is_busy (disk->dev)) {
-                if (ped_exception_throw (
+        if (ped_device_is_busy (disk->dev))
+                return ped_exception_throw (
                         PED_EXCEPTION_WARNING,
                         PED_EXCEPTION_IGNORE_CANCEL,
                         _("Partition(s) on %s are being used."),
-                        disk->dev->path)
-                                != PED_EXCEPTION_IGNORE)
-                        return 0;
-        }
+                        disk->dev->path) == PED_EXCEPTION_IGNORE;
+
         return 1;
 }
 
+static int
+_partition_warn_loss ()
+{
+        return ped_exception_throw (
+                PED_EXCEPTION_WARNING,
+                PED_EXCEPTION_YES_NO,
+                _("The existing file system will be destroyed and "
+                  "all data on the partition will be lost. Do "
+                  "you want to continue?"), 
+                NULL) == PED_EXCEPTION_YES;
+}
+
+static int
+_disk_warn_loss (PedDisk* disk)
+{
+        return ped_exception_throw (
+                PED_EXCEPTION_WARNING,
+                PED_EXCEPTION_YES_NO,
+                _("The existing disk label on %s will be destroyed "
+                  "and all data on this disk will be lost. Do you "
+                  "want to continue?"), 
+                disk->dev->path) == PED_EXCEPTION_YES;
+}
+
 /* This function changes "sector" to "new_sector" if the new value lies
  * within the required range.
  */
@@ -554,10 +580,12 @@ do_mklabel (PedDevice** dev)
         ped_exception_leave_all ();
 
         if (disk) {
-                if (!_disk_warn_busy (disk)) {
-                        ped_disk_destroy (disk);
-                        goto error;
-                }
+                if (!_disk_warn_busy (disk))
+                        goto error_destroy_disk;
+
+                if (!_disk_warn_loss (disk))
+                        goto error_destroy_disk;
+
                 ped_disk_destroy (disk);
         }
 
@@ -591,6 +619,9 @@ do_mkfs (PedDevice** dev)
         if (!disk)
                 goto error;
 
+        if  (!_partition_warn_loss())
+                goto error_destroy_disk;
+
         if (!command_line_get_partition (_("Partition number?"), disk, &part))
                 goto error_destroy_disk;
         if (!_partition_warn_busy (part))
@@ -1189,9 +1220,10 @@ do_print (PedDevice** dev)
         StrList*        row;
         int             has_extended;
         int             has_name;
-        int             has_num_arg = 0;
+        int             has_devices_arg = 0;
         int             has_free_arg = 0;
-        int             has_all_arg = 0;
+        int             has_list_arg = 0;
+        int             has_num_arg = 0;
         char*           transport[13] = {"unknown", "scsi", "ide", "dac960",
                                          "cpqarray", "file", "ataraid", "i2o",
                                          "ubd", "dasd", "viodasd", "sx8", "dm"};
@@ -1209,23 +1241,57 @@ do_print (PedDevice** dev)
 
         peek_word = command_line_peek_word ();
         if (peek_word) {
-                has_num_arg = isdigit (peek_word[0]);
-
-                if (strncmp (peek_word, "free", 4) == 0) {
+                if (strncmp (peek_word, "devices", 7) == 0) {
+                        command_line_pop_word();
+                        has_devices_arg = 1;
+                }
+                else if (strncmp (peek_word, "free", 4) == 0) {
                         command_line_pop_word ();
                         has_free_arg = 1;
                 } 
-
-                if (strncmp (peek_word, "all", 3) == 0) {
+                else if (strncmp (peek_word, "list", 4) == 0 ||
+                         strncmp (peek_word, "all", 3) == 0) {
                         command_line_pop_word();
-                        has_all_arg = 1;
+                        has_list_arg = 1;
                 }
+                else
+                        has_num_arg = isdigit(peek_word[0]);
 
                 ped_free (peek_word);
-
         }
 
-        if (has_num_arg) {
+        if (has_devices_arg) {
+                char*           dev_name;
+                PedDevice*      current_dev = NULL;
+
+                ped_device_probe_all();
+
+                while ((current_dev = ped_device_get_next(current_dev))) {
+                        end = ped_unit_format_byte (current_dev,
+                                             current_dev->length
+                                             * current_dev->sector_size);
+                        printf ("%s (%s)\n", current_dev->path, end);
+                        ped_free (end);
+                }    
+
+                dev_name = strdup ((*dev)->path);
+                ped_device_free_all ();
+
+                *dev = ped_device_get (dev_name);
+                if (!*dev)
+		        return 0;
+                if (!ped_device_open (*dev))
+                        return 0;
+
+                ped_free (dev_name);
+
+                return 1;
+        }
+
+        else if (has_list_arg) 
+                return _print_list (0);
+
+        else if (has_num_arg) {
                 PedPartition*   part = NULL;
                 int             status = 0;
                 if (command_line_get_partition ("", disk, &part))
@@ -1234,30 +1300,32 @@ do_print (PedDevice** dev)
                 return status;
         }
 
-        if (has_all_arg) {
-                int first_device;
-                PedDevice *current_dev;
-
-                ped_device_probe_all();
-                current_dev = NULL;
-
-                while ((current_dev = ped_device_get_next(current_dev))) {
-                        do_print (&current_dev);
-                        printf ("\n");
-                        current_dev = ped_device_get_next (current_dev);
-                }    
-
-                return 1;
-        }
-
         start = ped_unit_format (*dev, 0);
         end = ped_unit_format_byte (*dev, (*dev)->length * (*dev)->sector_size
                                           - 1 );
-        printf ("\n");
-        printf (_("Model: %s (%s)\n"), (*dev)->model, transport[(*dev)->type]);
-        printf (_("Disk %s: %s\n"), (*dev)->path, end);
-        printf (_("Sector size (logical/physical): %lldB/%lldB\n"),
-                        (*dev)->sector_size, (*dev)->phys_sector_size);
+        
+        if (opt_machine_mode) {
+            switch (ped_unit_get_default ()) {
+                case PED_UNIT_CHS:      printf ("CHS;\n");
+                                        break;
+                case PED_UNIT_CYLINDER: printf ("CYL;\n");
+                                        break;
+                default:                printf ("BYT;\n");
+                                        break;
+
+            }
+            printf ("%s:%s:%s:%lld:%lld:%s:%s;\n",
+                    (*dev)->path, end, transport[(*dev)->type],
+                    (*dev)->sector_size, (*dev)->phys_sector_size,
+                    disk->type->name, (*dev)->model);
+        } else {
+            printf (_("Model: %s (%s)\n"), 
+                    (*dev)->model, transport[(*dev)->type]);
+            printf (_("Disk %s: %s\n"), (*dev)->path, end);
+            printf (_("Sector size (logical/physical): %lldB/%lldB\n"),
+                    (*dev)->sector_size, (*dev)->phys_sector_size);
+        }
+
         ped_free (start);
         ped_free (end);
 
@@ -1267,15 +1335,23 @@ do_print (PedDevice** dev)
                 char* cyl_size = ped_unit_format_custom (*dev,
                                         chs->heads * chs->sectors,
                                         PED_UNIT_KILOBYTE);
-                printf (_("BIOS cylinder,head,sector geometry: %d,%d,%d.  "
-                          "Each cylinder is %s.\n"),
-                        chs->cylinders, chs->heads, chs->sectors, cyl_size);
+                
+                if (opt_machine_mode) {
+                    printf ("%d:%d:%d:%s;\n",
+                            chs->cylinders, chs->heads, chs->sectors, cyl_size);
+                } else {
+                    printf (_("BIOS cylinder,head,sector geometry: %d,%d,%d.  "
+                              "Each cylinder is %s.\n"),
+                            chs->cylinders, chs->heads, chs->sectors, cyl_size);
+                }
+
                 ped_free (cyl_size);
         }
 
-        printf (_("Partition Table: %s\n"), disk->type->name);
-
-        printf ("\n");
+        if (!opt_machine_mode) {
+            printf (_("Partition Table: %s\n"), disk->type->name);
+            printf ("\n");
+        }
         
         has_extended = ped_disk_type_check_feature (disk->type,
                                          PED_DISK_TYPE_EXTENDED);
@@ -1283,94 +1359,144 @@ do_print (PedDevice** dev)
                                          PED_DISK_TYPE_PARTITION_NAME);
 
         
-        if (ped_unit_get_default() == PED_UNIT_CHS) {
-                row = str_list_create (_("Number"), _("Start"),
-                                           _("End"), NULL);
+        if (!opt_machine_mode) {
+
+            if (ped_unit_get_default() == PED_UNIT_CHS) {
+                    row = str_list_create (_("Number"), _("Start"),
+                                               _("End"), NULL);
+            } else {
+                    row = str_list_create (_("Number"), _("Start"),
+                                               _("End"), _("Size"), NULL);
+            }
+
+            if (has_extended)
+                    str_list_append (row, _("Type"));
+
+            str_list_append (row, _("File system"));
+
+            if (has_name)
+                    str_list_append (row, _("Name"));
+
+            str_list_append (row, _("Flags"));
+
+
+            table = table_new (str_list_length(row));
+
+            table_add_row_from_strlist (table, row);
+
+
+            for (part = ped_disk_next_partition (disk, NULL); part;
+                 part = ped_disk_next_partition (disk, part)) {
+
+                    if ((!has_free_arg && !ped_partition_is_active(part)) ||
+                        part->type & PED_PARTITION_METADATA)
+                            continue;
+
+                    tmp = ped_malloc (4);
+
+                    if (part->num >= 0)
+                            sprintf (tmp, "%2d ", part->num);
+                    else
+                            sprintf (tmp, "%2s ", "");
+
+                    row = str_list_create (tmp, NULL);
+
+                    start = ped_unit_format (*dev, part->geom.start);
+                    end = ped_unit_format_byte (
+                            *dev,
+                            (part->geom.end + 1) * (*dev)->sector_size - 1);
+                    size = ped_unit_format (*dev, part->geom.length);
+                    if (ped_unit_get_default() == PED_UNIT_CHS) {
+                            str_list_append (row, start);
+                            str_list_append (row, end);
+                    } else {
+                            str_list_append (row, start);
+                            str_list_append (row, end);
+                            str_list_append (row, size);
+                    }
+
+                    if (!(part->type & PED_PARTITION_FREESPACE)) {
+                            if (has_extended) {
+                                name = 
+                                    _(ped_partition_type_get_name (part->type));
+                                str_list_append (row, name);
+                            }
+
+                            str_list_append (row, part->fs_type ?
+                                             part->fs_type->name : "");
+
+                            if (has_name) {
+                                    name = _(ped_partition_get_name (part));
+                                    str_list_append (row, name);
+                            }
+
+                            str_list_append (row, partition_print_flags (part));
+                    } else {
+                            if (has_extended)
+                                    str_list_append (row, "");
+                            str_list_append (row, _("Free Space"));
+                            if (has_name)
+                                    str_list_append (row, "");
+                            str_list_append (row, "");
+                    }
+
+                    //PED_ASSERT (row.cols == caption.cols)
+                    table_add_row_from_strlist (table, row);
+            }
+
+            table_rendered = table_render (table); 
+#ifdef ENABLE_NLS
+            printf("%ls\n", table_rendered);
+#else
+            printf("%s\n", table_rendered);
+#endif
+            ped_free (table_rendered);
+            table_destroy (table);
+
         } else {
-                row = str_list_create (_("Number"), _("Start"),
-                                           _("End"), _("Size"), NULL);
-        }
-
-        if (has_extended)
-                str_list_append (row, _("Type"));
-
-        str_list_append (row, _("File system"));
-
-        if (has_name)
-                str_list_append (row, _("Name"));
-
-        str_list_append (row, _("Flags"));
-
-
-        table = table_new (str_list_length(row));
-
-        table_add_row_from_strlist (table, row);
-
-
-        for (part = ped_disk_next_partition (disk, NULL); part;
-             part = ped_disk_next_partition (disk, part)) {
+    
+            for (part = ped_disk_next_partition (disk, NULL); part;
+                 part = ped_disk_next_partition (disk, part)) {
 
                 if ((!has_free_arg && !ped_partition_is_active(part)) ||
-                    part->type & PED_PARTITION_METADATA)
-                        continue;
-
-                tmp = ped_malloc (4);
-
+                        part->type & PED_PARTITION_METADATA)
+                            continue; 
+                
                 if (part->num >= 0)
-                        sprintf (tmp, "%2d ", part->num);
+                    printf ("%d:", part->num);
                 else
-                        sprintf (tmp, "%2s ", "");
+                    printf ("1:");
 
-                row = str_list_create (tmp, NULL);
-
-                start = ped_unit_format (*dev, part->geom.start);
-                end = ped_unit_format_byte (
-                        *dev, (part->geom.end + 1) * (*dev)->sector_size - 1);
-                size = ped_unit_format (*dev, part->geom.length);
-                if (ped_unit_get_default() == PED_UNIT_CHS) {
-                        str_list_append (row, start);
-                        str_list_append (row, end);
-                } else {
-                        str_list_append (row, start);
-                        str_list_append (row, end);
-                        str_list_append (row, size);
-                }
+                printf ("%s:", ped_unit_format (*dev, part->geom.start));
+                printf ("%s:", ped_unit_format_byte (
+                                *dev,
+                                (part->geom.end + 1) * 
+                                (*dev)->sector_size - 1));
 
+                if (ped_unit_get_default() != PED_UNIT_CHS)
+                    printf ("%s:", ped_unit_format (*dev,
+                                                    part->geom.length));
+                    
                 if (!(part->type & PED_PARTITION_FREESPACE)) {
-                        name = _(ped_partition_type_get_name (part->type));
 
-                        if (has_extended)
-                                str_list_append (row, name);
+                    if (part->fs_type)
+                        printf ("%s:", part->fs_type->name);
+                    else
+                        printf (":");
 
-                        str_list_append (row, part->fs_type ?
-                                         part->fs_type->name : "");
+                    if (has_name) 
+                        printf ("%s:", _(ped_partition_get_name (part)));
+                    else
+                        printf (":");
 
-                        if (has_name)
-                                str_list_append (row, name);
+                    printf ("%s;\n", partition_print_flags (part));
 
-                        str_list_append (row, partition_print_flags (part));
                 } else {
-                        if (has_extended)
-                                str_list_append (row, "");
-                        str_list_append (row, _("Free Space"));
-                        if (has_name)
-                                str_list_append (row, "");
-                        str_list_append (row, "");
+                    printf ("free;\n");
                 }
-
-                //PED_ASSERT (row.cols == caption.cols)
-                table_add_row_from_strlist (table, row);
+            }
         }
 
-        table_rendered = table_render (table); 
-#ifdef ENABLE_NLS
-        printf("%ls\n", table_rendered);
-#else
-        printf("%s\n", table_rendered);
-#endif
-        ped_free (table_rendered);
-
-        table_destroy (table);
         ped_disk_destroy (disk);
 
         return 1;
@@ -1381,6 +1507,24 @@ error:
         return 0;
 }
 
+static int
+_print_list (int cli)
+{
+        PedDevice *current_dev = NULL;
+
+        ped_device_probe_all();
+
+        while ((current_dev = ped_device_get_next(current_dev))) {
+                do_print (&current_dev);
+                printf ("\n");
+        }    
+
+        if(cli)
+                exit(0);
+
+        return 1;
+}
+
 static int
 do_quit (PedDevice** dev)
 {
@@ -1758,7 +1902,7 @@ do_version ()
 {
     printf ("\n%s\n%s",
             prog_name,
-            copyright_msg);
+            _(copyright_msg));
     return 1;
 }
 
@@ -1923,10 +2067,10 @@ NULL),
                 NULL, 1));
 
         command_register (commands, command_create (
-                str_list_create_unique ("mklabel", _("mklabel"), NULL),
+                str_list_create_unique ("mklabel", _("mklabel"), "mktable", _("mktable"), NULL),
                 do_mklabel,
                 str_list_create (
-_("mklabel LABEL-TYPE                       create a new disklabel "
+_("mklabel,mktable LABEL-TYPE               create a new disklabel "
   "(partition table)"),
 NULL),
                 str_list_create (label_type_msg, NULL), 1));
@@ -1959,7 +2103,7 @@ NULL), 1));
                 do_mkpartfs,
                 str_list_create (
 _("mkpartfs PART-TYPE FS-TYPE START END     make a partition with a "
-"file system"),
+  "file system"),
 NULL),
         str_list_create (_(part_type_msg), _(start_end_msg), NULL), 1));
 
@@ -1983,15 +2127,17 @@ command_register (commands, command_create (
         str_list_create_unique ("print", _("print"), NULL),
         do_print,
         str_list_create (
-_("print [free|NUMBER|all]                  display the partition table, "
+_("print [devices|free|list,all|NUMBER]     display the partition table, "
   "a partition, or all devices"),
 NULL),
         str_list_create (
-_("Without arguments, print displays the entire partition table. With 'free'\n"
-"argument, information about free space will be displayed otherwise if a\n"
-"partition number is given, then more detailed information is displayed\n"
-"about that partition. If the 'all' argument is passed instead, partition\n"
-"information for all devices will be displayed."),
+_("Without arguments, print displays the entire partition table. However "
+  "with the following arguments it performs the various other actions.\n"),
+_("a. devices : display all active block devices\n"),
+_("b. free    : display information about free unpartitioned space on the "
+  "current block device\n"),
+_("c. list,all: display partition tables of all active block devices\n"),
+_("d. NUMBER  : display more detail information about particular partition\n"),
 NULL), 1));
 
 command_register (commands, command_create (
@@ -2115,17 +2261,18 @@ int     opt;
 while (1)
 {
 #ifdef HAVE_GETOPT_H
-        opt = getopt_long (*argc_ptr, *argv_ptr, "hisv",
+        opt = getopt_long (*argc_ptr, *argv_ptr, "hilmsv",
                            options, NULL);
 #else
-        opt = getopt (*argc_ptr, *argv_ptr, "hisv");
+        opt = getopt (*argc_ptr, *argv_ptr, "hilmsv");
 #endif
         if (opt == -1)
                 break;
 
         switch (opt) {
                 case 'h': help_msg (); break;
-                case 'i': opt_script_mode = 0; break;
+                case 'l': _print_list(1); break;
+                case 'm': opt_machine_mode = 1; break;
                 case 's': opt_script_mode = 1; break;
                 case 'v': _version (); break;
         }
@@ -2188,6 +2335,14 @@ _init_commands ();
 
 if (!_parse_options (argc_ptr, argv_ptr))
         goto error_done_commands;
+
+#ifdef HAVE_GETUID
+        if (getuid() != 0) {
+            printf(_("WARNING: You are not superuser.  Watch out for "
+                     "permissions.\n"));
+        }
+#endif
+
 dev = _choose_device (argc_ptr, argv_ptr);
 if (!dev)
         goto error_done_commands;
@@ -2219,7 +2374,7 @@ if (dev->boot_dirty && dev->type != PED_DEVICE_FILE) {
           "rebooting.  Read section 4 of the Parted User "
           "documentation for more information."));
 }
-if (dev->type != PED_DEVICE_FILE && !opt_script_mode) {
+if (dev->type != PED_DEVICE_FILE && !opt_script_mode && !opt_machine_mode) {
         ped_exception_throw (
                 PED_EXCEPTION_INFORMATION, PED_EXCEPTION_OK,
                 _("Don't forget to update /etc/fstab, if "
@@ -2240,13 +2395,6 @@ main (int argc, char** argv)
         PedDevice*      dev;
         int             status;
 
-#ifdef HAVE_GETUID
-        if (getuid() != 0) {
-            printf(_("WARNING: You are not superuser.  Watch out for "
-                     "permissions.\n"));
-        }
-#endif
-
         dev = _init (&argc, &argv);
         if (!dev)
                 return 1;
